var Tools = {
    getLocalStorage: function(key) {
        var listen = window.localStorage.getItem('listen');
        if (listen) {
            listen = JSON.parse(listen);
            return listen[key];
        } else {
            return null;
        }
    },
    setLocalStorage: function(key, value) {
        var listen;
        if (window.localStorage.getItem('listen')) {
            listen = JSON.parse(window.localStorage.getItem('listen'));
        } else {
            listen = {};
        }
        listen[key] = value;
        window.localStorage.setItem('listen', JSON.stringify(listen));
    },
    removeLocalStorage: function(key) {
        var listen = JSON.parse(window.localStorage.getItem('listen'));
        listen[key] = '';
        window.localStorage.setItem('listen', JSON.stringify(listen));
    }
};
var SoftWare = {
    'fr': {
        'productid': 1
    },
    'de': {
        'productid': 9
    },
    'es': {
        'productid': 31
    },
    'en': {
        'productid': 23
    }
}
var Qt = {
    queryState: function(key) {
        ParagraphApi.queryState(key);
    },
    paraSetState: function(type, state) {
        ParagraphApi.paraSetState(type, state);
    },
    setHideDisplayTipIndex: function(index) {
        ParagraphApi.setHideDisplayTipIndex(index);
    },
    setDragState: function(state) {
        ParagraphApi.setDragState(state);
    },
    captureWord: function(url) {
        ParagraphApi.captureWord(url);
    },
    fillwordSuccessAction: function() {
        ParagraphApi.paragraphFillwordSuccess();
    }
};
var Paragraph = function($container, $control, $paragraph, $translation, $dictation) {
    this.lang = '';
    this.$container = $container;
    this.$control = $control;
    this.$displayTip = $container.find('.displayTip');
    this.$content = $container.find('#content');
    this.$pre = $container.find('#pre');
    this.$playControl = $container.find($('#playControl'));
    this.$next = $container.find('#next');
    this.$transControl = $container.find($('#transControl'));
    this.$repeatControl = $container.find($('#repeatControl'));
    this.$modeControl = $container.find($('#modeControl'));
    this.$desktopControl = $container.find($('#desktopControl'));
    this.explainTipState = Tools.getLocalStorage('explainTipState');
    this.translateState = Tools.getLocalStorage('translateState');
    this.repeatState = Tools.getLocalStorage('repeatState');
    this.modeState = Tools.getLocalStorage('modeState');
    this.desktopState = Tools.getLocalStorage('desktopState');
    this.$paragraph = $paragraph;
    this.$translation = $translation;
    this.$dictation = $dictation;
    this.choisedIndex = 0;
    this.choisedItem = 9;
    this.choisedState = false;
    this.$fillwordEscape = [];
    this.$choised = $container.find('#choised');
    this.defaultParagraph = '暂无文章';
    this.defaultTranslation = '{"chinese":"暂无译文","index":0,"state":false}';
    this.dragTimeout = '';
    this.dragState = false;
    this.init();
    this.captureWord();
};
Paragraph.prototype.setLang = function(lang) {
    this.lang = lang;
}
Paragraph.prototype.captureWord = function() {
    var curPosX, curPosY, curCaptured = false,
        maxWordsLeft = 2,
        maxWordsRight = 5;
    var timeout, self = this;
    $('#paragraph').bind('mouseout', function() {
        clearTimeout(timeout);
    });
    $('#paragraph').bind('mousemove', function(e) {
        clearTimeout(timeout);
        curPosX = e.clientX;
        curPosY = e.clientY;
        curCaptured = false;
        timeout = setTimeout(function() {
            tryCapture();
        }, 500);
    });

    function isSeprator(s) {
        var bCn = /[^\u4e00-\u9fa5]/.test(s);
        return (!bCn || s == ' ');
    }

    function tryCapture() {
        if (curCaptured) {
            return;
        }
        var r = document.caretRangeFromPoint(curPosX, curPosY);
        if (!r || !r.startContainer.data || (r.startOffset == 0 && r.endOffset == 0) || r.endOffset == nodeLength) {
            return;
        }
        var nodeLength = r.startContainer.length;
        curCaptured = true;
        var midPos = r.startOffset;
        var startPos = r.startOffset - 1;
        var endPos = r.endOffset;
        var cLeft = cRight = 0;
        var capturedLine = '';

        var textContainer = r.startContainer;
        if (textContainer.nodeType == 3) {
            capturedLine = textContainer.textContent;
        } else {
            capturedLine = textContainer.innerText;
        }
        r.detach();
        for (; startPos >= 0; startPos--) {
            if (isSeprator(capturedLine.substring(startPos, startPos + 1))) {
                cLeft++;
            }
            if (cLeft > maxWordsLeft) {
                break;
            }
        }
        for (; endPos <= nodeLength; endPos++) {
            if (isSeprator(capturedLine.substring(endPos - 1, endPos))) {
                cRight++;
            }
            if (cRight > maxWordsRight) {
                break;
            }
        }
        var requestString = capturedLine.substring(startPos, endPos);
        var requestPostion = (midPos - startPos - 2);
        if ((requestPostion + 3) < requestString.length) {
            var productId = 32071 + SoftWare[self.lang]['productid'];
            requestString = encodeURIComponent(requestString);
            var url = 'http://127.0.0.1:' + productId + '/eudic_capture_word?word=' + requestString + '&curpos=' + requestPostion + '&src=ting_desktop';
            Qt.captureWord(url);
        }
    }
};
Paragraph.prototype.init = function() {
    var self = this;
    if (!this.explainTipState) {
        Qt.queryState('explainTipState');
    }
    if (!this.translateState) {
        Qt.queryState('translateState');
    }
    if (!this.repeatState) {
        Qt.queryState('repeatState');
    }
    if (!this.modeState) {
        Qt.queryState('modeState');
    }
    if (!this.desktopState) {
        Qt.queryState('desktopState');
    }
    this.setModeState();
    this.$displayTip.click(function() {
        var index = $(this).parent().attr('index');
        self.$translation.addClass('showTrans')
        Qt.setHideDisplayTipIndex(index);
    });
    this.$pre.click(function() {
        self.$playControl.find('a').attr('class', 'play');
        Qt.paraSetState('play', 'pre');
    });
    this.$playControl.click(function() {
        var state;
        switch ($(this).find('a').attr('class')) {
            case 'play':
                state = 'pause';
                break;
            case 'pause':
                state = 'play';
                break;
        }
        $(this).find('a').attr('class', state);
        Qt.paraSetState('play', state);
    });
    this.$next.click(function() {
        self.$playControl.find('a').attr('class', 'play');
        Qt.paraSetState('play', 'next');
    });
    this.$transControl.click(function() {
        Qt.paraSetState('translateState', '');
    });
    this.$repeatControl.click(function() {
        Qt.paraSetState('repeatState', '');
    });
    this.$modeControl.click(function() {
        Qt.paraSetState('modeState', '');
    });
    this.$desktopControl.click(function() {
        Qt.paraSetState('desktopState', '');
    });
    this.dragAction();
    this.setParagraph(this.defaultParagraph);
    this.setDictTrans(this.defaultTranslation);
    this.$choised.click(function(e) {
        var $target = $(e.target);
        if ($target.hasClass('cword')) {
            self.choisedAction($target);
        }
    });
};
Paragraph.prototype.dragAction = function() {
    var self = this,
        leaveElem;
    this.$container.bind('mouseover', function() {
        $(this).addClass('hover');
        self.$control.css({
            'visibility': 'visible'
        });
    }).bind('mouseout', function(e) {
        leaveElem = e.toElement;
        if (!leaveElem || leaveElem.nodeName.toLowerCase() == 'body') {
            $(this).removeClass('hover');
            self.$control.css({
                'visibility': 'hidden'
            });
        }
    }).bind('mouseup', function() {
        self.dragState = false;
        $('body').removeClass('notSelect');
    });
    this.$paragraph.bind('mousedown', function() {
        self.dragState = true;
        $('body').addClass('notSelect');
        self.$control.css({
            'visibility': 'hidden'
        });
        Qt.setDragState('start');
    });
};
Paragraph.prototype.saveState = function(type, state) {
    switch (type) {
        case 'hasTranslation':
            if (state == 'true') {
                $('body').removeClass('noTrans');
            }
            if (state == 'false') {
                $('body').addClass('noTrans');
            }
            break;
        case 'explainTipState':
            this.explainTipState = state;
            Tools.setLocalStorage('explainState', this.explainTipState);
            this.setExplainTipState();
            break;
        case 'translateState':
            this.translateState = state;
            Tools.setLocalStorage('translateState', this.translateState);
            this.setTranslateState();
            break;
        case 'repeatState':
            this.repeatState = state;
            Tools.setLocalStorage('repeatState', this.repeatState);
            this.setRepeatState();
            break;
        case 'modeState':
            this.modeState = state;
            Tools.setLocalStorage('modeState', this.modeState);
            this.setModeState();
            break;
        case 'desktopState':
            this.desktopState = state;
            Tools.setLocalStorage('desktopState', this.desktopState);
            this.setDesktopState();
            break;
    }
};
Paragraph.prototype.setParagraph = function(paragraph) {
    paragraph = paragraph.replace(/======/g, "'").replace(/;;;;;;/g, '"');
    this.$paragraph.html(paragraph);
};
Paragraph.prototype.setDictTrans = function(dictTrans) {
    dictTrans = JSON.parse(dictTrans);
    var chinese = dictTrans['chinese'];
    chinese = chinese.replace(/======/g, "'").replace(/;;;;;;/g, '"');
    this.$translation.find('.chinese').text(chinese);
    if (dictTrans['state']) {
        this.$translation.attr('class', 'showTrans');
    } else {
        this.$translation.attr('class', '');
    }
    this.$translation.attr('index', dictTrans['index']);
};
Paragraph.prototype.setChoised = function(choised) {
    this.$choised.html(choised);
};
Paragraph.prototype.setFillword = function(fillword) {
    fillword = JSON.parse(fillword);
    this.$paragraph.html(fillword['text'].replace(/======/g, "'").replace(/;;;;;;/g, '"'));
    this.choisedItem = fillword['item'];
    this.choisedIndex = 0;
    this.choisedState = true;
    this.$fillwordEscape = this.$paragraph.find('.escape');
};
Paragraph.prototype.setPlayState = function(state) {
    if (state == 'play') {
        this.$playControl.find('a').attr('class', 'play');
    }
    if (state == 'pause') {
        this.$playControl.find('a').attr('class', 'pause');
    }
};
Paragraph.prototype.setExplainTipState = function() {
    var state = this.explainTipState;
    switch (state) {
        case 'highlight':
            $('body').addClass('transHighlightSet');
            break;
        case 'explain':
            $('body').removeClass('transHighlightSet');
            break;
    }
};
Paragraph.prototype.setTranslateState = function() {
    var state = this.translateState.split(',')[0];
    this.$transControl.find('a').attr('class', state);
    switch (state) {
        case 'translateOn':
            this.$container.removeClass('hideLrc');
            this.$content.removeClass('showTip');
            break;
        case 'translateTap':
            this.$container.removeClass('hideLrc');
            this.$content.addClass('showTip');
            break;
        case 'translateOff':
            this.$container.addClass('hideLrc');
            break;
    }
};
Paragraph.prototype.setRepeatState = function() {
    var state = this.repeatState.split(',')[0];
    this.$repeatControl.find('a').attr('class', state);
};
Paragraph.prototype.setModeState = function() {
    var state = this.modeState.split(',')[0];
    this.$modeControl.find('a').attr('class', state);
    switch (state) {
        case 'read':
            this.$choised.hide();
            this.$paragraph.insertBefore(this.$translation);
            break;
        case 'fillword':
            this.$choised.show();
            this.$translation.insertBefore(this.$paragraph);
            break;
    }
};
Paragraph.prototype.setDesktopState = function() {
    var state = this.desktopState.split(',')[0];
    this.$desktopControl.find('a').attr('class', state);
};
Paragraph.prototype.choisedAction = function($target) {
    var choisedWord = $target.text(),
        $correct = this.$fillwordEscape.eq(0),
        correctWord = $correct.text();
    if (choisedWord == correctWord) {
        if (this.choisedState) {
            $correct.addClass('correct');
        } else {
            $correct.addClass('error');
            this.choisedState = true;
        }
        $correct.addClass('fill').removeClass('escape');
        this.$fillwordEscape = $('.escape');
        this.choisedIndex++;
        if (this.choisedIndex == this.choisedItem) {
            $correct.addClass('correct');
            setTimeout(function() {
                Qt.fillwordSuccessAction();
            }, 1000);
        }
    } else {
        this.choisedState = false;
        $target.addClass('animError');
        setTimeout(function() {
            $target.removeClass('animError');
        }, 500);
    }
};